<?php

/**
 * Функции модуля typofilter
 */

require 'typofilter.engine.inc';

/**
 * Реализует hook_help
 * @param string $section - секция, для которой написан хелп
 * @return text - текст хелпа
 */
function typofilter_help($section) {
	$output = "";
	switch ($section) {
		case 'admin/help#typofilter':
			$output .= '<p>';
			$output .= 'Модуль содержит типографский фильтр.';
			$output .= 'Фильтр позволяет автоматически ставить тире вместо дефисов (там, где надо), ';
			$output .= 'елочки вместо знаков дюйма (") и т.п. ';
			$output .= '</p>';
			$output .= '<p>';
			$output .= 'Фильтр обладает страницей настройки, на которой можно ';
			$output .= 'включать/выключать те или иные замены, а также задавать ';
			$output .= 'безопасные блоки, внутри которых фильтр работать не будет.';
			$output .= '</p>';
			$output .= '<p>';
			$output .= 'В качестве движка фильтра используется немного измененный "<a href="http://rmcreative.ru/blog/post/tipograf">Типограф</a>".';
			$output .= '</p>';
			break;
	}
	
	return $output;
} //function typofilter_help


/**
 * Реализует hook_perm, задает разрешение администрировать фильтр
 *
 * @return array - массив разрешений
 */
function typofilter_perm() {
	return array('administer typofilter');
} //function typofilter_perm

/**
 * Реализует hook_menu
 *
 * @return array - массив вводимых модулем элементов меню
 */
function typofilter_menu() {
	$items = array();
	
	//страница админки
	$items['admin/settings/typofilter'] = array(
		'title' => 'Настроить типографский фильтр',
		'description' => 'Страница настройки опций и безопасных блоков типографского фильтра',
		'page callback' => 'drupal_get_form',
		'page arguments' => array('typofilter_admin_settings'),
		'access arguments' => array('administer typofilter'),
		'type' => MENU_NORMAL_ITEM,
	);
	
	return $items;
} //function typofilter_menu()


/**
 * Реализует hook_filter, в котором объявляет наличие фильтра в модуле
 *
 * @param string $op - операция, list/description/prepare/process
 * @param int $delta - номер фильтра 
 * @param int $format - формат
 * @param text $text - обрабатываемый текст
 * @param int $cache_id - ID кэш
 * @return text для всех операций кроме list (для list - массив фильтров)
 */
function typofilter_filter($op,$delta=0,$format=-1,$text='',$cache_id=0) {
	switch ($op) {
		case 'list':
			return array(0 => t('Typofilter'),);
			
		case 'description':
			return t('Редактирует текст с применением типографских символов..');
			
		case 'prepare':
			return $text;
			
		case 'process':
			$filter = new Typographus_Lite_UTF8();
			//добавляем сейфблоки
			$safeblocks = variable_get('typofilter_safe_blocks',array());
			foreach ($safeblocks as $open => $close) {
				$filter->addSafeBlock($open,$close);
			}
			//обрабатываем текст
			$options = variable_get('typofilter_options',array());
			$text = $filter->process($text,$options);
			return $text;
			
		default:
			return $text;
	}
} //function typofilter_filter


/**
 * Создает форму админки
 *
 * @return array - массив формы
 */
function typofilter_admin_settings() {
	$form = array();
	
	//опции фильтрации
	$form['filter_options'] = array(
		'#type'  => 'checkboxes',
		'#title' => t('Опции фильтрации'),
		'#default_value' => _get_filter_options_checkboxes(),
		'#options' => array(
			'fix_spaces' => t('Исправить множественные пробелы'),
			'replace_copyright' => t('Заменить (c) на ©'),
			'replace_registered' => t('Заменить (r) на ®'),
			'replace_trademark' => t('Заменить (tm) на ™'),
			'replace_hellip' => t('Заменить точки на многоточия'),
			'replace_plusminus' => t('Заменить +- на &plusmn;'),
			'fix_quotes' => t('Исправить кавычки'),
			'nested_quotes' => t('Исправить вложенные кавычки'),
			'fix_dashes' => t('Исправить тире'),
			'fix_ndash' => t('Короткое тире в числовых диапазонах'),
			'glue_percent' => t('Знак % без пробела от числа'),
			'glue_IP' => t('Склеить IP'),
		),
	);
	
	//безопасные блоки
	$form['safe_block'] = array(
		'#type'  => 'fieldset',
		'#title' => t('Безопасные блоки'),
		'#collapsible' => FALSE,
		'#tree' => TRUE,
	);
	
	//Существующие блоки
	$safe_blocks = variable_get('typofilter_safe_blocks',array());
	$safeblockstr = "<ol>";
	foreach ($safe_blocks as $left => $right) {
		$safeblockstr .= "<li>" . check_plain($left) ." " .check_plain($right) . "</li>";
	}
	$safeblockstr .= "</ol>";
	$form['safe_block']['safe_blocks_exist'] = array(
		'#type' => 'item',
		'#title' => t('Используются блоки'),
		'#value' => $safeblockstr,
	);
		
	$form['safe_block']['opentag'] = array(
		'#type'  => 'textfield',
		'#title' => t('Открывающий тег'),
		'#default_value' => '',
	);
	$form['safe_block']['closetag'] = array(
		'#type'  => 'textfield',
		'#title' => t('Закрывающий тег'),
		'#default_value' => '',
	);
	
	//кнопка сброса пользовательских тегов
	$form['buttons']['clear_blocks'] = array(
		'#type'	  => 'submit',
		'#value'  => t('Сбросить блоки'),
		'#submit' => array('_clear_safe_blocks'),
	);
	//скопировано из system_settings_form чтобы использовать свою функцию сабмишна
	$form['buttons']['submit'] = array('#type' => 'submit', '#value' => t('Save configuration') );
	$form['buttons']['reset'] = array('#type' => 'submit', '#value' => t('Reset to defaults') );

	if (!empty($_POST) && form_get_errors()) {
		drupal_set_message(t('The settings have not been saved because of the errors.'), 'error');
	}
	
	$form['#theme'] = 'system_settings_form';
	
	return $form;
} //function typofilter_admin_settings


/**
 * Валидирует данные формы админки
 *
 * @param array $form - массив формы
 * @param array $form_state - ссылка на массив состояний формы
 */ 
function typofilter_admin_settings_validate($form,&$form_state) {
	//валидировать пока нечего
} //function typofilter_admin_settings_validate


/**
 * Сохраняет данные формы админки модуля
 *
 * @param array $form - массив формы
 * @param array $form_state - ссылка на массив состояний формы
 */
function typofilter_admin_settings_submit($form,&$form_state) {
	$op = isset($form_state['values']['op']) ? $form_state['values']['op'] : '';

	// Удаляем ненужные элементы
	unset($form_state['values']['submit'], $form_state['values']['reset'], $form_state['values']['form_id'], $form_state['values']['op'], $form_state['values']['form_token'], $form_state['values']['form_build_id']);

	if ($op == t('Reset to defaults')) {
		//сбрасываем все переменные в умолчания
		_clear_safe_blocks();
		_reset_filter_options();
		drupal_set_message(t('The configuration options have been reset to their default values.'));
    } else {
    	//устанавливаем переменные
    	//опции фильтра
    	$filter_options = array_filter($form_state['values']['filter_options']);
    	$saved_options = array();
    	foreach ($filter_options as $option) {
    		$saved_options[$option] = 1;
    	}
    	variable_set('typofilter_options',$saved_options);
    	//блоки
		$safe_blocks = variable_get('typofilter_safe_blocks',array());
		//сохраняем блок только если не пустые поля ввода
		if (($form_state['values']['safe_block']['opentag'] != "") && ($form_state['values']['safe_block']['closetag'] != "")) {
			$safe_blocks[$form_state['values']['safe_block']['opentag']] = $form_state['values']['safe_block']['closetag'];
		}
    	variable_set('typofilter_safe_blocks', $safe_blocks);
		drupal_set_message(t('The configuration options have been saved.'));
    }	
	
	cache_clear_all();
  	drupal_rebuild_theme_registry();
} //function typofilter_admin_settings_submit


/**
 * Удаляет безопасные блоки, установленные пользователем
 *
 */
function _clear_safe_blocks() {
	$safe_blocks = array(
		'<pre[^>]*>' => '<\/pre>',
		'<style[^>]*>' => '<\/style>',
		'<script[^>]*>' => '<\/script>',
		'<!--' => '-->',
	    '<code[^>]*>' => '<\/code>',
		'<safeblock>' => '<\/safeblock>',
	);
	variable_set('typofilter_safe_blocks',$safe_blocks);	
} //function _clear_safe_blocks


/**
 * Возвращает массив с именами установленных опций в качестве значений для использования в чекбоксах
 *
 * @return Array - массив установленных опций
 */
function _get_filter_options_checkboxes() {
	$filter_options = variable_get('typofilter_options',array());
	$plain_options = array();
	foreach ($filter_options as $key => $value) {
		$plain_options[] = $key;
	}
	return $plain_options;
} //function __get_filter_options_checkboxes


/**
 * Сбрасывает в умолчания опции фильтра (считаем что по-умолчанию все разрешено)
 * 
 */
function _reset_filter_options() {
	$filter_options = array(
		'fix_spaces' => 1,
		'replace_copyright' => 1,
		'replace_registered' => 1,
		'replace_trademark' => 1,
		'replace_hellip' => 1,
		'replace_plusminus' => 1,
		'fix_quotes' => 1,
		'nested_quotes' => 1,
		'fix_dashes' => 1,
		'fix_ndash' => 1,
		'glue_percent' => 1,
		'glue_IP' => 1,
	);
	variable_set('typofilter_options',$filter_options);
} //function _reset_filter_options
